"""Top-level package for fmp_tcc."""

__author__ = """Bayan Saparbayeva, AlexandreBayan Saparbayeva, Alexandre Courtois Auberger, Bayan Saparbayeva, and Vish Patel"""
__email__ = 'saparbayevabt@gmail.com'
__version__ = '0.1.0'
